package v1alpha1

import (
	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
)

// Subnets type contains relevant networks related to a remote cluster.
type Subnets struct {
	// Network used in the remote cluster for local Pods. Default is "None": this means remote cluster uses local cluster PodCIDR.
	LocalNATPodCIDR string `json:"localNATPodCIDR"`
	// Network used for Pods in the remote cluster.
	RemotePodCIDR string `json:"remotePodCIDR"`
	// Network used in remote cluster for local service endpoints. Default is "None": this means remote cluster uses local cluster ExternalCIDR.
	LocalNATExternalCIDR string `json:"localNATExternalCIDR"`
	// Network used in local cluster for remote service endpoints.
	RemoteExternalCIDR string `json:"remoteExternalCIDR"`
}

// ClusterMapping is an empty struct.
type ClusterMapping struct{}

// ConfiguredCluster is an empty struct used as value for NatMappingsConfigured.
type ConfiguredCluster struct{}

// EndpointMapping describes a relation between an enpoint IP and an IP belonging to ExternalCIDR.
type EndpointMapping struct {
	// IP belonging to cluster ExtenalCIDR assigned to this endpoint.
	IP string `json:"ip"`
	// Set of clusters to which this endpoint has been reflected. Only the key, which is the ClusterID, is useful.
	ClusterMappings map[string]ClusterMapping `json:"clusterMappings"`
}

// IpamSpec defines the desired state of Ipam.
type IpamSpec struct {
	// Map consumed by go-ipam module. Key is prefic cidr, value is a Prefix.
	Prefixes map[string][]byte `json:"prefixes"`
	// Network pools.
	Pools []string `json:"pools"`
	// Map used to keep track of networks assigned to clusters. Key is the remote cluster ID, value is a the set of
	// networks used by the remote cluster.
	ClusterSubnets map[string]Subnets `json:"clusterSubnets"`
	// Cluster ExternalCIDR
	ExternalCIDR string `json:"externalCIDR"`
	// Endpoint IP mappings. Key is the IP address of the local endpoint, value is the IP of the remote endpoint, so it belongs to an ExternalCIDR
	EndpointMappings map[string]EndpointMapping `json:"endpointMappings"`
	// NatMappingsConfigured is a map that contains all the remote clusters
	// for which NatMappings have been already configured.
	// Key is a cluster ID, value is an empty struct.
	NatMappingsConfigured map[string]ConfiguredCluster `json:"natMappingsConfigured"`
	// Cluster PodCIDR
	PodCIDR string `json:"podCIDR"`
	// ServiceCIDR
	ServiceCIDR string `json:"serviceCIDR"`
}

// IpamStorage is the Schema for the ipams API.
type IpamStorage struct {
	metav1.TypeMeta   `json:",inline"`
	metav1.ObjectMeta `json:"metadata,omitempty"`

	Spec IpamSpec `json:"spec,omitempty"`
}

// +kubebuilder:object:root=true

// IpamStorageList contains a list of Ipam.
type IpamStorageList struct {
	metav1.TypeMeta `json:",inline"`
	metav1.ListMeta `json:"metadata,omitempty"`
	Items           []IpamStorage `json:"items"`
}

func init() {
	SchemeBuilder.Register(&IpamStorage{}, &IpamStorageList{})
}
