package v1alpha1

import (
	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
)

// NatMappingSpec defines the desired state of NatMapping.
type NatMappingSpec struct {
	// ClusterID is the cluster this resource refers to.
	ClusterID string `json:"clusterID"`
	// PodCIDR is the network used for remote pods in the local cluster.
	// It can be either the RemotePodCIDR or the RemoteNATPodCIDR.
	PodCIDR string `json:"podCIDR"`
	// ExternalCIDR is the ExternalCIDR used in the remote cluster for local exported resource.
	// It can be either the LocalExternalCIDR or the LocalNATExternalCIDR.
	ExternalCIDR string `json:"externalCIDR"`
	// ClusterMappings is the set of NAT mappings currently active.
	ClusterMappings Mappings `json:"clusterMappings"`
}

// Mappings is the type describing a set mappings for a remote cluster. Key is the old IP, value is the new NAT-ted IP.
type Mappings map[string]string

// NatMappingStatus defines the observed state of NatMapping.
type NatMappingStatus struct {
}

// NatMapping is the Schema for the natmappings API.
type NatMapping struct {
	metav1.TypeMeta   `json:",inline"`
	metav1.ObjectMeta `json:"metadata,omitempty"`

	Spec   NatMappingSpec   `json:"spec,omitempty"`
	Status NatMappingStatus `json:"status,omitempty"`
}

// NatMappingList contains a list of NatMapping.
type NatMappingList struct {
	metav1.TypeMeta `json:",inline"`
	metav1.ListMeta `json:"metadata,omitempty"`
	Items           []NatMapping `json:"items"`
}

func init() {
	SchemeBuilder.Register(&NatMapping{}, &NatMappingList{})
}
