package v1alpha1

import (
	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	"k8s.io/apimachinery/pkg/runtime/schema"

	crdclient "github.com/liqotech/liqo/pkg/crdClient"
)

// NetworkConfigSpec defines the desired state of NetworkConfig.
type NetworkConfigSpec struct {
	// The ID of the remote cluster that will receive this CRD.
	ClusterID string `json:"clusterID"`
	// Network used in the local cluster for the pod IPs.
	PodCIDR string `json:"podCIDR"`
	// Network used for local service endpoints.
	ExternalCIDR string `json:"externalCIDR"`
	// Public IP of the node where the VPN tunnel is created.
	EndpointIP string `json:"endpointIP"`
	// Vpn technology used to interconnect two clusters.
	BackendType string `json:"backendType"`
	// Connection parameters
	BackendConfig map[string]string `json:"backend_config"`
}

// NetworkConfigStatus defines the observed state of NetworkConfig.
type NetworkConfigStatus struct {
	// Indicates if this network config has been processed by the remote cluster.
	// +kubebuilder:default=false
	Processed bool `json:"processed"`
	// The new subnet used to NAT the podCidr of the remote cluster. The original PodCidr may have been mapped to this
	// network by the remote cluster.
	PodCIDRNAT string `json:"podCIDRNAT,omitempty"`
	// The new subnet used to NAT the externalCIDR of the remote cluster. The original ExternalCIDR may have been mapped
	// to this network by the remote cluster.
	ExternalCIDRNAT string `json:"externalCIDRNAT,omitempty"`
}

type NetworkConfig struct {
	metav1.TypeMeta   `json:",inline"`
	metav1.ObjectMeta `json:"metadata,omitempty"`

	Spec   NetworkConfigSpec   `json:"spec,omitempty"`
	Status NetworkConfigStatus `json:"status,omitempty"`


// NetworkConfigList contains a list of NetworkConfig.
type NetworkConfigList struct {
	metav1.TypeMeta `json:",inline"`
	metav1.ListMeta `json:"metadata,omitempty"`
	Items           []NetworkConfig `json:"items"`
}

func init() {
	SchemeBuilder.Register(&NetworkConfig{}, &NetworkConfigList{})

	crdclient.AddToRegistry("networkconfigs", &NetworkConfig{}, &NetworkConfigList{}, nil, schema.GroupResource{
		Group:    TunnelEndpointGroupResource.Group,
		Resource: "networkconfigs",
	})
}
